/*
    This file is part of karl-marx.

    karl-marx is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    karl-marx is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with karl-marx.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.congressodeiradicali.karlmarx

import com.bot4s.telegram.methods.{KickChatMember, UnbanChatMember}
import com.bot4s.telegram.models._

class BotUser(private val user: User,
              private val chat: Chat,
              private val bot: Bot,
              private val chatMember: ChatMember) {

  def ban() : Unit = {
    // ban works by sending the kick user request
    bot.request(KickChatMember(ChatId.fromChat(chat.id), user.id))
  }

  def kick() : Unit = {
    // kick works by kicking and then unbanning the user
    bot.request(KickChatMember(ChatId.fromChat(chat.id), user.id))
    bot.request(UnbanChatMember(ChatId.fromChat(chat.id), user.id))
  }


  def isCreator: Boolean = chatMember.status == MemberStatus.Creator
  def isAdmin: Boolean = chatMember.status == MemberStatus.Administrator
  def isMember: Boolean = chatMember.status == MemberStatus.Member
  def isKicked: Boolean = chatMember.status == MemberStatus.Kicked

  def canBanUsers : Boolean = chatMember.canRestrictMembers.getOrElse(false)


}
