/*
    This file is part of karl-marx.

    karl-marx is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    karl-marx is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with karl-marx.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.congressodeiradicali.karlmarx

import com.bot4s.telegram.api.declarative.CommandFilterMagnet
import com.bot4s.telegram.models.Message

import scala.concurrent.Future

trait Plugin {
  type MessageHandler = Message => Future[Option[String]]
  type CommandHandler = (Message, Array[String]) => Future[Option[String]]
  type CommandHandlerPair = (CommandFilterMagnet, CommandHandler)

  // Do not set this manually, it is handled by PluginManager
  var enabled = false

  val name: String
  val description: String
  val license: String
  val author: String

  /**
    * A string that identifies the plugin
    * used for loading and unloading it with bot commands
    */
  val identifier: String

  var messageHandlers: List[MessageHandler]
  var commandHandlers: Map[CommandFilterMagnet, CommandHandler]
}