/*
    This file is part of karl-marx.

    karl-marx is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    karl-marx is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with karl-marx.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.congressodeiradicali.karlmarx.coreplugins

import com.bot4s.telegram.api.declarative.{CommandFilterMagnet, CommandImplicits}
import org.congressodeiradicali.karlmarx._

import scala.concurrent.Future

class BanPlugin(bot: Bot) extends CorePlugin with CommandImplicits {

  override val name: String = "Ban Plugin"
  override val description: String = "Ban users and unban them"

  override val identifier: String = "ban"

  override var commandHandlers: Map[CommandFilterMagnet, CommandHandler] = Map {
    stringToCommandFilter("ban") -> { (msg, _) =>

      val futureLocalizedString = msg.replyToMessage.fold {
        Future { LocalizableString.BAN_FAILED_REPLY }
      } { reply =>
        reply.from.fold {
          Future { LocalizableString.BAN_FAILED_INVALID_USER }
        } { user =>
          bot.requestBotUser(user, msg.chat).map { target =>
            if (target.isAdmin || target.isCreator)
              LocalizableString.BAN_FAILED_BAN_ADMIN
            else {
              target.ban()
              LocalizableString.BAN_SUCCESSFUL
            }
          }
        }
      }

      futureLocalizedString.map { s => Some(bot.localize(s)) }
    }
  }
  override var messageHandlers: List[MessageHandler] = List()
}
