/*
    This file is part of karl-marx.

    karl-marx is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    karl-marx is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with karl-marx.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.congressodeiradicali.karlmarx.coreplugins

import com.bot4s.telegram.api.declarative.{CommandFilterMagnet, CommandImplicits}
import com.bot4s.telegram.models.Message
import org.congressodeiradicali.karlmarx.{Bot, LocalizableString}

import scala.concurrent.Future

class PluginManagerPlugin(bot: Bot) extends CorePlugin with CommandImplicits {
  // This is enabled by default for obvious reasons - users need to be able to enable other plugins on the first start.
  // Todo: disable it when reasonable
  enabled = true

  override val name = "Plugin Manager"
  override val description = "Manages other plugins"

  override val identifier = "pluginmanager"

  def enableByIdentifier(id: String): Boolean =
    if (bot.plugins.contains(id)) {
      bot.plugins(id).enabled = true
      true
    } else false

  def disableByIdentifier(id: String): Boolean =
    if (bot.plugins.keySet.contains(id)) {
      bot.plugins(id).enabled = false
      true
    } else false

  override var commandHandlers: Map[CommandFilterMagnet, CommandHandler] = Map(
    stringToCommandFilter("enable") -> { (_: Message, argv: Array[String]) =>
      val name = argv(1)
      val response = if (enableByIdentifier(name))
      LocalizableString.PLUGIN_ENABLED
      else
      LocalizableString.NO_SUCH_PLUGIN

      Future { Some(bot.localize(response)) }
    },

    stringToCommandFilter("disable") -> { (_: Message, argv: Array[String]) =>
      val name = argv(1)
      val response = if (disableByIdentifier(name))
        LocalizableString.PLUGIN_DISABLED
      else
        LocalizableString.NO_SUCH_PLUGIN

      Future { Some(bot.localize(response)) }
    },

    stringToCommandFilter("listplugins") -> { (_, _) =>
      Future {
        Some(
          bot.plugins.values.map { plugin =>
            s"${plugin.identifier}: ${plugin.name}. ${plugin.description}. License: ${plugin.license}. Author: ${plugin.author}"
          }.mkString("\n\n")
        )
      }
    },

    stringToCommandFilter("listenabled") -> { (_, _) =>
      Future {
        Some(
          bot.plugins.values.filter(_.enabled).map(_.identifier).mkString("\n")
        )
      }
    },
  )
  override var messageHandlers: List[MessageHandler] = List()
}
