/*
    This file is part of karl-marx.

    karl-marx is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    karl-marx is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with karl-marx.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.congressodeiradicali.karlmarx.coreplugins

import com.bot4s.telegram.api.declarative.{CommandFilterMagnet, CommandImplicits}
import com.bot4s.telegram.models.Message
import org.congressodeiradicali.karlmarx.{Bot, LocalizableString}

import scala.concurrent.Future

class SetPlugin(bot: Bot) extends CorePlugin with CommandImplicits {
  override val name: String = "Set Plugin"
  override val description: String = "A plugin to set responses to triggers."

  override val identifier: String = "set"

  override var commandHandlers: Map[CommandFilterMagnet, CommandHandler] = Map(
    stringToCommandFilter("set") -> { (_, argv) =>
      val trigger = argv(1)
      val response = argv.drop(2).mkString(" ") // Drop command + trigger
      bot.redis.set("PLUGIN_SET_" + trigger, response).map { _ =>
        Some(bot.localize(LocalizableString.DONE))
      }
    }
  )
  override var messageHandlers: List[MessageHandler] = List(
    { msg: Message =>
      msg.text.map { text =>
        val firstWord = text.takeWhile(_ != ' ')
        bot.redis.get("PLUGIN_SET_" + firstWord)
      }.getOrElse(Future { None })
    }
  )
}
